@@ -1,27 +1,35 @@
 #![no_std]
 #![no_main]
+#![feature(abi_efiapi)]
 #![feature(custom_test_frameworks)]
 #![test_runner(libertyos_kernel::testexec)]
 #![reexport_test_harness_main = "testexec"]
 #![allow(dead_code)]
 #![allow(deprecated)]
 #![allow(unused_features)]
+#![allow(unused_imports)]
+#![allow(unused_mut)]
+#![allow(unused_variables)]
 
+extern crate alloc;
 
+use alloc::{boxed::Box, vec, vec::Vec, rc::Rc};
 use bootloader::{BootInfo, entry_point};
 use core::panic::PanicInfo;
 use libertyos_kernel::println;
+use uefi::prelude::*;
+use uefi::ResultExt;
 
 entry_point!(kernel_main);
 
-
 fn kernel_main(bootinfo: &'static BootInfo) -> !
 {
 	use libertyos_kernel::mem::{self, BootInfoFrameAllocator};
+	use libertyos_kernel::allocator;
 	use x86_64::{structures::paging::Page, VirtAddr};
 
 	println!("LIBERTY-OS");
-	println!("KERNEL VERSION 0.10.0");
+	println!("KERNEL VERSION 0.11.2");
 	libertyos_kernel::init();
 
 	let physmem_offset = VirtAddr::new(bootinfo.physical_memory_offset);
@@ -38,10 +46,26 @@ fn kernel_main(bootinfo: &'static BootInfo) -> !
 	libertyos_kernel::mem::new_example_mapping(page, &mut mapper, &mut framealloc);
 
 	let page_ptr: *mut u64 = page.start_address().as_mut_ptr();
-	unsafe
-	{
-		page_ptr.offset(400).write_volatile(0x_f021_f077_f065_f04e)
-	};
+//	unsafe
+//	{
+//		page_ptr.offset(400).write_volatile(0x_f021_f077_f065_f04e)
+//	};
+
+	allocator::init_heap(&mut mapper, &mut framealloc)
+		.expect("[ERROR] FAILED TO INITIALIZE HEAP");
+
+	let heap_value = Box::new(41);
+	println!("[INFO] HEAP_VALUE AT {:p}", heap_value);
+
+	let mut vec = (0..500).collect::<Vec<i32>>();
+	
+	println!("[INFO] VEC AT {:p}", vec.as_slice());
+
+	let refcounted = Rc::new(vec![1, 2, 3]);
+	let clonedref = refcounted.clone();
+	println!("[INFO] CURRENT REFERENCE COUNT: {}", Rc::strong_count(&clonedref));
+	core::mem::drop(refcounted);
+	println!("[INFO] CURRENT REFERENCE COUNT: {}", Rc::strong_count(&clonedref));
 
 	#[cfg(test)]
 	testexec();
